@@ -23,27 +23,43 @@
 # CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
+import os
 import webbrowser
 import datetime
 from datetime import date
 from typing import TYPE_CHECKING, Tuple, Dict
 import threading
 from enum import IntEnum
+from decimal import Decimal
+
+from PyQt5.QtGui import QMouseEvent, QFont, QBrush, QColor
+from PyQt5.QtCore import (Qt, QPersistentModelIndex, QModelIndex, QAbstractItemModel,
+                          QSortFilterProxyModel, QVariant, QItemSelectionModel, QDate, QPoint)
+from PyQt5.QtWidgets import (QMenu, QHeaderView, QLabel, QMessageBox,
+                             QPushButton, QComboBox, QVBoxLayout, QCalendarWidget,
+                             QGridLayout)
 
 from electrum.address_synchronizer import TX_HEIGHT_LOCAL
 from electrum.i18n import _
-from electrum.util import (block_explorer_URL, profiler, print_error, TxMinedInfo,
-                           OrderedDictWithIndex, PrintError)
+from electrum.util import (block_explorer_URL, profiler, TxMinedInfo,
+                           OrderedDictWithIndex, timestamp_to_datetime)
+from electrum.logging import get_logger, Logger
 
-from .util import *
+from .util import (read_QIcon, MONOSPACE_FONT, Buttons, CancelButton, OkButton,
+                   filename_field, MyTreeView, AcceptFileDragDrop, WindowModalDialog,
+                   CloseButton)
 
 if TYPE_CHECKING:
     from electrum.wallet import Abstract_Wallet
 
+
+_logger = get_logger(__name__)
+
+
 try:
     from electrum.plot import plot_history, NothingToPlotException
 except:
-    print_error("qt/history_list: could not import electrum.plot. This feature needs matplotlib to be installed.")
+    _logger.info("could not import electrum.plot. This feature needs matplotlib to be installed.")
     plot_history = None
 
 # note: this list needs to be kept in sync with another in kivy
@@ -77,18 +93,25 @@ class HistorySortModel(QSortFilterProxyModel):
         item2 = self.sourceModel().data(source_right, Qt.UserRole)
         if item1 is None or item2 is None:
             raise Exception(f'UserRole not set for column {source_left.column()}')
-        if item1.value() is None or item2.value() is None:
+        v1 = item1.value()
+        v2 = item2.value()
+        if v1 is None or isinstance(v1, Decimal) and v1.is_nan(): v1 = -float("inf")
+        if v2 is None or isinstance(v2, Decimal) and v2.is_nan(): v2 = -float("inf")
+        try:
+            return v1 < v2
+        except:
             return False
-        return item1.value() < item2.value()
 
-class HistoryModel(QAbstractItemModel, PrintError):
+class HistoryModel(QAbstractItemModel, Logger):
 
     def __init__(self, parent):
-        super().__init__(parent)
+        QAbstractItemModel.__init__(self, parent)
+        Logger.__init__(self)
         self.parent = parent
         self.view = None  # type: HistoryList
         self.transactions = OrderedDictWithIndex()
         self.tx_status_cache = {}  # type: Dict[str, Tuple[int, str]]
+        self.summary = None
 
     def set_view(self, history_list: 'HistoryList'):
         # FIXME HistoryModel and HistoryList mutually depend on each other.
@@ -142,7 +165,7 @@ class HistoryModel(QAbstractItemModel, PrintError):
             return QVariant(d[col])
         if role not in (Qt.DisplayRole, Qt.EditRole):
             if col == HistoryColumns.STATUS_ICON and role == Qt.DecorationRole:
-                return QVariant(self.view.icon_cache.get(":icons/" +  TX_ICONS[status]))
+                return QVariant(read_QIcon(TX_ICONS[status]))
             elif col == HistoryColumns.STATUS_ICON and role == Qt.ToolTipRole:
                 return QVariant(str(conf) + _(" confirmation" + ("s" if conf != 1 else "")))
             elif col > HistoryColumns.DESCRIPTION and role == Qt.TextAlignmentRole:
@@ -152,7 +175,7 @@ class HistoryModel(QAbstractItemModel, PrintError):
                 return QVariant(monospace_font)
             elif col == HistoryColumns.DESCRIPTION and role == Qt.DecorationRole \
                     and self.parent.wallet.invoices.paid.get(tx_hash):
-                return QVariant(self.view.icon_cache.get(":icons/seal"))
+                return QVariant(read_QIcon("seal"))
             elif col in (HistoryColumns.DESCRIPTION, HistoryColumns.COIN_VALUE) \
                     and role == Qt.ForegroundRole and tx_item['value'].value < 0:
                 red_brush = QBrush(QColor("#BC1E1E"))
@@ -207,7 +230,7 @@ class HistoryModel(QAbstractItemModel, PrintError):
 
     @profiler
     def refresh(self, reason: str):
-        self.print_error(f"refreshing... reason: {reason}")
+        self.logger.info(f"refreshing... reason: {reason}")
         assert self.parent.gui_thread == threading.current_thread(), 'must be called from GUI thread'
         assert self.view, 'view not set'
         selected = self.view.selectionModel().currentIndex()
@@ -232,9 +255,7 @@ class HistoryModel(QAbstractItemModel, PrintError):
         self.endInsertRows()
         if selected_row:
             self.view.selectionModel().select(self.createIndex(selected_row, 0), QItemSelectionModel.Rows | QItemSelectionModel.SelectCurrent)
-        f = self.view.current_filter
-        if f:
-            self.view.filter(f)
+        self.view.filter()
         # update summary
         self.summary = r['summary']
         if not self.view.years and self.transactions:
@@ -283,6 +304,7 @@ class HistoryModel(QAbstractItemModel, PrintError):
             'confirmations':  tx_mined_info.conf,
             'timestamp':      tx_mined_info.timestamp,
             'txpos_in_block': tx_mined_info.txpos,
+            'date':           timestamp_to_datetime(tx_mined_info.timestamp),
         })
         topLeft = self.createIndex(row, 0)
         bottomRight = self.createIndex(row, len(HistoryColumns) - 1)
@@ -464,26 +486,26 @@ class HistoryList(MyTreeView, AcceptFileDragDrop):
         grid = QGridLayout()
         grid.addWidget(QLabel(_("Start")), 0, 0)
         grid.addWidget(QLabel(self.format_date(start_date)), 0, 1)
-        grid.addWidget(QLabel(str(h.get('start_fiat_value')) + '/BTC'), 0, 2)
+        grid.addWidget(QLabel(str(h.get('fiat_start_value')) + '/BTC'), 0, 2)
         grid.addWidget(QLabel(_("Initial balance")), 1, 0)
         grid.addWidget(QLabel(format_amount(h['start_balance'])), 1, 1)
-        grid.addWidget(QLabel(str(h.get('start_fiat_balance'))), 1, 2)
+        grid.addWidget(QLabel(str(h.get('fiat_start_balance'))), 1, 2)
         grid.addWidget(QLabel(_("End")), 2, 0)
         grid.addWidget(QLabel(self.format_date(end_date)), 2, 1)
-        grid.addWidget(QLabel(str(h.get('end_fiat_value')) + '/BTC'), 2, 2)
+        grid.addWidget(QLabel(str(h.get('fiat_end_value')) + '/BTC'), 2, 2)
         grid.addWidget(QLabel(_("Final balance")), 4, 0)
         grid.addWidget(QLabel(format_amount(h['end_balance'])), 4, 1)
-        grid.addWidget(QLabel(str(h.get('end_fiat_balance'))), 4, 2)
+        grid.addWidget(QLabel(str(h.get('fiat_end_balance'))), 4, 2)
         grid.addWidget(QLabel(_("Income")), 5, 0)
-        grid.addWidget(QLabel(format_amount(h.get('income'))), 5, 1)
-        grid.addWidget(QLabel(str(h.get('fiat_income'))), 5, 2)
+        grid.addWidget(QLabel(format_amount(h.get('incoming'))), 5, 1)
+        grid.addWidget(QLabel(str(h.get('fiat_incoming'))), 5, 2)
         grid.addWidget(QLabel(_("Expenditures")), 6, 0)
-        grid.addWidget(QLabel(format_amount(h.get('expenditures'))), 6, 1)
-        grid.addWidget(QLabel(str(h.get('fiat_expenditures'))), 6, 2)
+        grid.addWidget(QLabel(format_amount(h.get('outgoing'))), 6, 1)
+        grid.addWidget(QLabel(str(h.get('fiat_outgoing'))), 6, 2)
         grid.addWidget(QLabel(_("Capital gains")), 7, 0)
-        grid.addWidget(QLabel(str(h.get('capital_gains'))), 7, 2)
+        grid.addWidget(QLabel(str(h.get('fiat_capital_gains'))), 7, 2)
         grid.addWidget(QLabel(_("Unrealized gains")), 8, 0)
-        grid.addWidget(QLabel(str(h.get('unrealized_gains', ''))), 8, 2)
+        grid.addWidget(QLabel(str(h.get('fiat_unrealized_gains', ''))), 8, 2)
         vbox.addLayout(grid)
         vbox.addLayout(Buttons(CloseButton(d)))
         d.setLayout(vbox)
@@ -529,7 +551,7 @@ class HistoryList(MyTreeView, AcceptFileDragDrop):
             self.show_transaction(tx_item['txid'])
 
     def show_transaction(self, tx_hash):
-        tx = self.wallet.transactions.get(tx_hash)
+        tx = self.wallet.db.get_transaction(tx_hash)
         if not tx:
             return
         label = self.wallet.get_label(tx_hash) or None # prefer 'None' if not defined (force tx dialog to hide Description field if missing)
@@ -550,7 +572,9 @@ class HistoryList(MyTreeView, AcceptFileDragDrop):
             column_title = self.hm.headerData(column, Qt.Horizontal, Qt.DisplayRole)
             column_data = self.hm.data(idx, Qt.DisplayRole).value()
         tx_hash = tx_item['txid']
-        tx = self.wallet.transactions[tx_hash]
+        tx = self.wallet.db.get_transaction(tx_hash)
+        if not tx:
+            return
         tx_URL = block_explorer_URL(self.config, 'tx', tx_hash)
         height = self.wallet.get_tx_height(tx_hash).height
         is_relevant, is_mine, v, fee = self.wallet.get_wallet_delta(tx)
@@ -559,7 +583,12 @@ class HistoryList(MyTreeView, AcceptFileDragDrop):
         menu = QMenu()
         if height == TX_HEIGHT_LOCAL:
             menu.addAction(_("Remove"), lambda: self.remove_local_tx(tx_hash))
+
+        amount_columns = [HistoryColumns.COIN_VALUE, HistoryColumns.RUNNING_COIN_BALANCE, HistoryColumns.FIAT_VALUE, HistoryColumns.FIAT_ACQ_PRICE, HistoryColumns.FIAT_CAP_GAINS]
+        if column in amount_columns:
+            column_data = column_data.strip()
         menu.addAction(_("Copy {}").format(column_title), lambda: self.parent.app.clipboard().setText(column_data))
+
         for c in self.editable_columns:
             if self.isColumnHidden(c): continue
             label = self.hm.headerData(c, Qt.Horizontal, Qt.DisplayRole)
@@ -577,7 +606,7 @@ class HistoryList(MyTreeView, AcceptFileDragDrop):
                 if child_tx:
                     menu.addAction(_("Child pays for parent"), lambda: self.parent.cpfp(tx, child_tx))
         if pr_key:
-            menu.addAction(self.icon_cache.get(":icons/seal"), _("View invoice"), lambda: self.parent.show_invoice(pr_key))
+            menu.addAction(read_QIcon("seal"), _("View invoice"), lambda: self.parent.show_invoice(pr_key))
         if tx_URL:
             menu.addAction(_("View on block explorer"), lambda: webbrowser.open(tx_URL))
         menu.exec_(self.viewport().mapToGlobal(position))
@@ -587,15 +616,14 @@ class HistoryList(MyTreeView, AcceptFileDragDrop):
         to_delete |= self.wallet.get_depending_transactions(delete_tx)
         question = _("Are you sure you want to remove this transaction?")
         if len(to_delete) > 1:
-            question = _(
-                "Are you sure you want to remove this transaction and {} child transactions?".format(len(to_delete) - 1)
-            )
+            question = (_("Are you sure you want to remove this transaction and {} child transactions?")
+                        .format(len(to_delete) - 1))
         answer = QMessageBox.question(self.parent, _("Please confirm"), question, QMessageBox.Yes, QMessageBox.No)
         if answer == QMessageBox.No:
             return
         for tx in to_delete:
             self.wallet.remove_transaction(tx)
-        self.wallet.save_transactions(write=True)
+        self.wallet.storage.write()
         # need to update at least: history_list, utxo_list, address_list
         self.parent.need_update.set()
 
